@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006 by Peter Penz (peter.penz@gmx.at)                  *
+ *   Copyright (C) 2006-2009 by Peter Penz <peter.penz@gmx.at>             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -21,30 +21,30 @@
 
 #include "dolphincategorydrawer.h"
 #include "dolphincontroller.h"
-#include "dolphinsettings.h"
-#include "dolphinviewautoscroller.h"
+#include "settings/dolphinsettings.h"
+#include "dolphinsortfilterproxymodel.h"
 #include "dolphin_iconsmodesettings.h"
 #include "dolphin_generalsettings.h"
 #include "draganddrophelper.h"
 #include "selectionmanager.h"
+#include "viewextensionsfactory.h"
 #include "zoomlevelinfo.h"
 
 #include <kcategorizedsortfilterproxymodel.h>
 #include <kdialog.h>
-#include <kdirmodel.h>
 #include <kfileitemdelegate.h>
 
 #include <QAbstractProxyModel>
 #include <QApplication>
 #include <QScrollBar>
 
-DolphinIconsView::DolphinIconsView(QWidget* parent, DolphinController* controller) :
+DolphinIconsView::DolphinIconsView(QWidget* parent,
+                                   DolphinController* controller,
+                                   DolphinSortFilterProxyModel* proxyModel) :
     KCategorizedView(parent),
-    m_enableScrollTo(false),
     m_controller(controller),
-    m_selectionManager(0),
-    m_autoScroller(0),
     m_categoryDrawer(0),
+    m_extensionsFactory(0),
     m_font(),
     m_decorationSize(),
     m_decorationPosition(QStyleOptionViewItem::Top),
@@ -53,6 +53,7 @@ DolphinIconsView::DolphinIconsView(QWidget* parent, DolphinController* controlle
     m_dropRect()
 {
     Q_ASSERT(controller != 0);
+    setModel(proxyModel);
     setLayoutDirection(Qt::LeftToRight);
     setViewMode(QListView::IconMode);
     setResizeMode(QListView::Adjust);
@@ -62,7 +63,6 @@ DolphinIconsView::DolphinIconsView(QWidget* parent, DolphinController* controlle
     viewport()->setAcceptDrops(true);
 
     setMouseTracking(true);
-    m_autoScroller = new DolphinViewAutoScroller(this);
 
     connect(this, SIGNAL(clicked(const QModelIndex&)),
             controller, SLOT(requestTab(const QModelIndex&)));
@@ -74,14 +74,6 @@ DolphinIconsView::DolphinIconsView(QWidget* parent, DolphinController* controlle
                 controller, SLOT(triggerItem(const QModelIndex&)));
     }
 
-    if (DolphinSettings::instance().generalSettings()->showSelectionToggle()) {
-        m_selectionManager = new SelectionManager(this);
-        connect(m_selectionManager, SIGNAL(selectionChanged()),
-                this, SLOT(requestActivation()));
-        connect(m_controller, SIGNAL(urlChanged(const KUrl&)),
-                m_selectionManager, SLOT(reset()));
-    }
-
     connect(this, SIGNAL(entered(const QModelIndex&)),
             controller, SLOT(emitItemEntered(const QModelIndex&)));
     connect(this, SIGNAL(viewportEntered()),
@@ -109,7 +101,6 @@ DolphinIconsView::DolphinIconsView(QWidget* parent, DolphinController* controlle
     }
 
     setWordWrap(settings->numberOfTextlines() > 1);
-    updateGridSize(view->showPreview(), 0);
 
     if (settings->arrangement() == QListView::TopToBottom) {
         setFlow(QListView::LeftToRight);
@@ -126,8 +117,11 @@ DolphinIconsView::DolphinIconsView(QWidget* parent, DolphinController* controlle
 
     setFocus();
 
-    connect(KGlobalSettings::self(), SIGNAL(kdisplayFontChanged()),
-            this, SLOT(updateFont()));
+    connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)),
+            this, SLOT(slotGlobalSettingsChanged(int)));
+
+    updateGridSize(view->showPreview(), 0);
+    m_extensionsFactory = new ViewExtensionsFactory(this, controller);
 }
 
 DolphinIconsView::~DolphinIconsView()
@@ -136,19 +130,6 @@ DolphinIconsView::~DolphinIconsView()
     m_categoryDrawer = 0;
 }
 
-void DolphinIconsView::scrollTo(const QModelIndex& index, ScrollHint hint)
-{
-    // Enable the QListView implementation of scrollTo() only if it has been
-    // triggered by a key press. Otherwise QAbstractItemView wants to scroll to the current
-    // index each time the layout has been changed. This becomes an issue when
-    // previews are loaded and the scrollbar is used: the scrollbar will always
-    // be reset to 0 on each new preview.
-    if (m_enableScrollTo || (state() != QAbstractItemView::NoState)) {
-        KCategorizedView::scrollTo(index, hint);
-        m_enableScrollTo = false;
-    }
-}
-
 void DolphinIconsView::dataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight)
 {
     KCategorizedView::dataChanged(topLeft, bottomRight);
@@ -165,6 +146,7 @@ QStyleOptionViewItem DolphinIconsView::viewOptions() const
 {
     QStyleOptionViewItem viewOptions = KCategorizedView::viewOptions();
     viewOptions.font = m_font;
+    viewOptions.fontMetrics = QFontMetrics(m_font);
     viewOptions.decorationPosition = m_decorationPosition;
     viewOptions.decorationSize = m_decorationSize;
     viewOptions.displayAlignment = m_displayAlignment;
@@ -191,14 +173,8 @@ void DolphinIconsView::mousePressEvent(QMouseEvent* event)
         setState(QAbstractItemView::DraggingState);
     }
 
-    if (!index.isValid()) {
-        if (QApplication::mouseButtons() & Qt::MidButton) {
-            m_controller->replaceUrlByClipboard();
-        }
-        const Qt::KeyboardModifiers modifier = QApplication::keyboardModifiers();
-        if (!(modifier & Qt::ShiftModifier) && !(modifier & Qt::ControlModifier)) {
-            clearSelection();
-        }
+    if (!index.isValid() && (QApplication::mouseButtons() & Qt::MidButton)) {
+         m_controller->replaceUrlByClipboard();
     }
 
     KCategorizedView::mousePressEvent(event);
@@ -206,9 +182,6 @@ void DolphinIconsView::mousePressEvent(QMouseEvent* event)
 
 void DolphinIconsView::startDrag(Qt::DropActions supportedActions)
 {
-    // TODO: invoking KCategorizedView::startDrag() should not be necessary, we'll
-    // fix this in KDE 4.1
-    KCategorizedView::startDrag(supportedActions);
     DragAndDropHelper::instance().startDrag(this, supportedActions, m_controller);
 }
 
@@ -255,31 +228,93 @@ void DolphinIconsView::dropEvent(QDropEvent* event)
     const QModelIndex index = indexAt(event->pos());
     const KFileItem item = m_controller->itemForIndex(index);
     m_controller->indicateDroppedUrls(item, m_controller->url(), event);
+    // don't call KCategorizedView::dropEvent(event), as it moves
+    // the items which is not wanted
+}
 
-    KCategorizedView::dropEvent(event);
+QModelIndex DolphinIconsView::moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers)
+{
+    const QModelIndex oldCurrent = currentIndex();
+
+    QModelIndex newCurrent = KCategorizedView::moveCursor(cursorAction, modifiers);
+    if (newCurrent != oldCurrent) {
+        return newCurrent;
+    }
+
+    // The cursor has not been moved by the base implementation. Provide a
+    // wrap behavior, so that the cursor will go to the next item when reaching
+    // the border.
+    const IconsModeSettings* settings = DolphinSettings::instance().iconsModeSettings();
+    if (settings->arrangement() == QListView::LeftToRight) {
+        switch (cursorAction) {
+        case MoveUp:
+            if (newCurrent.row() == 0) {
+                return newCurrent;
+            }
+            newCurrent = KCategorizedView::moveCursor(MoveLeft, modifiers);
+            selectionModel()->setCurrentIndex(newCurrent, QItemSelectionModel::NoUpdate);
+            newCurrent = KCategorizedView::moveCursor(MovePageDown, modifiers);
+            break;
+
+        case MoveDown:
+            if (newCurrent.row() == (model()->rowCount() - 1)) {
+                return newCurrent;
+            }
+            newCurrent = KCategorizedView::moveCursor(MovePageUp, modifiers);
+            selectionModel()->setCurrentIndex(newCurrent, QItemSelectionModel::NoUpdate);
+            newCurrent = KCategorizedView::moveCursor(MoveRight, modifiers);
+            break;
+
+        default:
+            break;
+        }
+    } else {
+        QModelIndex current = oldCurrent;
+        switch (cursorAction) {
+        case MoveLeft:
+            if (newCurrent.row() == 0) {
+                return newCurrent;
+            }
+            newCurrent = KCategorizedView::moveCursor(MoveUp, modifiers);
+            do {
+                selectionModel()->setCurrentIndex(newCurrent, QItemSelectionModel::NoUpdate);
+                current = newCurrent;
+                newCurrent = KCategorizedView::moveCursor(MoveRight, modifiers);
+            } while (newCurrent != current);
+            break;
+
+        case MoveRight:
+            if (newCurrent.row() == (model()->rowCount() - 1)) {
+                return newCurrent;
+            }
+            do {
+                selectionModel()->setCurrentIndex(newCurrent, QItemSelectionModel::NoUpdate);
+                current = newCurrent;
+                newCurrent = KCategorizedView::moveCursor(MoveLeft, modifiers);
+            } while (newCurrent != current);
+            newCurrent = KCategorizedView::moveCursor(MoveDown, modifiers);
+            break;
+
+        default:
+            break;
+        }
+    }
+
+    // Revert all changes of the current item to make sure that item selection works correctly
+    selectionModel()->setCurrentIndex(oldCurrent, QItemSelectionModel::NoUpdate);
+    return newCurrent;
 }
 
 void DolphinIconsView::keyPressEvent(QKeyEvent* event)
 {
-    m_enableScrollTo = true; // see DolphinIconsView::scrollTo()
     KCategorizedView::keyPressEvent(event);
     m_controller->handleKeyPressEvent(event);
 }
 
 void DolphinIconsView::wheelEvent(QWheelEvent* event)
 {
-    if (m_selectionManager != 0) {
-        m_selectionManager->reset();
-    }
-
-    // let Ctrl+wheel events propagate to the DolphinView for icon zooming
-    if (event->modifiers() & Qt::ControlModifier) {
-        event->ignore();
-        return;
-    }
-
-    horizontalScrollBar()->setSingleStep(m_itemSize.width() / 10);
-    verticalScrollBar()->setSingleStep(m_itemSize.height() / 10);
+    horizontalScrollBar()->setSingleStep(m_itemSize.width() / 5);
+    verticalScrollBar()->setSingleStep(m_itemSize.height() / 5);
 
     KCategorizedView::wheelEvent(event);
     // if the icons are aligned left to right, the vertical wheel event should
@@ -317,9 +352,7 @@ void DolphinIconsView::leaveEvent(QEvent* event)
 void DolphinIconsView::currentChanged(const QModelIndex& current, const QModelIndex& previous)
 {
     KCategorizedView::currentChanged(current, previous);
-    if (current.isValid() && !m_autoScroller->isActive()) {
-        scrollTo(current);
-    }
+    m_extensionsFactory->handleCurrentIndexChange(current, previous);
 }
 
 void DolphinIconsView::resizeEvent(QResizeEvent* event)
@@ -371,14 +404,23 @@ void DolphinIconsView::requestActivation()
     m_controller->requestActivation();
 }
 
-void DolphinIconsView::updateFont()
+void DolphinIconsView::slotGlobalSettingsChanged(int category)
 {
+    Q_UNUSED(category);
+
     const IconsModeSettings* settings = DolphinSettings::instance().iconsModeSettings();
     Q_ASSERT(settings != 0);
-
     if (settings->useSystemFont()) {
         m_font = KGlobalSettings::generalFont();
     }
+
+    disconnect(this, SIGNAL(clicked(QModelIndex)), m_controller, SLOT(triggerItem(QModelIndex)));
+    disconnect(this, SIGNAL(doubleClicked(QModelIndex)), m_controller, SLOT(triggerItem(QModelIndex)));
+    if (KGlobalSettings::singleClick()) {
+        connect(this, SIGNAL(clicked(QModelIndex)), m_controller, SLOT(triggerItem(QModelIndex)));
+    } else {
+        connect(this, SIGNAL(doubleClicked(QModelIndex)), m_controller, SLOT(triggerItem(QModelIndex)));
+    }
 }
 
 void DolphinIconsView::updateGridSize(bool showPreview, int additionalInfoCount)
@@ -442,16 +484,12 @@ void DolphinIconsView::updateGridSize(bool showPreview, int additionalInfoCount)
     }
 
     m_itemSize = QSize(itemWidth, itemHeight);
-    setGridSize(QSize(itemWidth + spacing * 2, itemHeight + spacing));
+    setGridSizeOwn(QSize(itemWidth + spacing * 2, itemHeight + spacing));
 
     KFileItemDelegate* delegate = dynamic_cast<KFileItemDelegate*>(itemDelegate());
     if (delegate != 0) {
         delegate->setMaximumSize(m_itemSize);
     }
-
-    if (m_selectionManager != 0) {
-        m_selectionManager->reset();
-    }
 }
 
 int DolphinIconsView::additionalInfoCount() const
