@@ -19,6 +19,8 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
+#include <QDate>
+#include <kdebug.h>
 #include "widgets.h"
 
 #include <kmessagebox.h>
@@ -62,10 +64,10 @@ class WinIdEmbedder: public QObject
 {
 public:
     WinIdEmbedder(bool printID, WId winId):
-        QObject(kapp), print(printID), id(winId)
+        QObject(qApp), print(printID), id(winId)
     {
-        if (kapp)
-            kapp->installEventFilter(this);
+        if (qApp)
+            qApp->installEventFilter(this);
     }
 protected:
     bool eventFilter(QObject *o, QEvent *e);
@@ -97,9 +99,9 @@ bool WinIdEmbedder::eventFilter(QObject *o, QEvent *e)
 bool sendVisualNotification(QString text, QString title, int timeout)
 {
 #ifdef QT_QTDBUS_FOUND
-  const QString dbusServiceName = "org.kde.VisualNotifications";
-  const QString dbusInterfaceName = "org.kde.VisualNotifications";
-  const QString dbusPath = "/VisualNotifications";
+  const QString dbusServiceName = "org.freedesktop.Notifications";
+  const QString dbusInterfaceName = "org.freedesktop.Notifications";
+  const QString dbusPath = "/org/freedesktop/Notifications";
 
   // check if service already exists on plugin instantiation
   QDBusConnectionInterface* interface = QDBusConnection::sessionBus().interface();
@@ -117,7 +119,6 @@ bool sendVisualNotification(QString text, QString title, int timeout)
 
   args.append("kdialog"); // app_name
   args.append(0U); // replaces_id
-  args.append("kdialogPassivePopup"); // event_id
   args.append("dialog-information"); // app_icon
   args.append(title); // summary
   args.append(text); // body
@@ -291,7 +292,7 @@ static int directCommand(KCmdLineArgs *args)
       if (args->count() > 0)
           init = args->arg(0);
 
-      bool retcode = Widgets::inputBox(0, title, args->getOption("inputbox"), init, result);
+      const bool retcode = Widgets::inputBox(0, title, args->getOption("inputbox"), init, result);
       cout << result.toLocal8Bit().data() << endl;
       return retcode ? 0 : 1;
     }
@@ -301,7 +302,7 @@ static int directCommand(KCmdLineArgs *args)
     if (args->isSet("password"))
     {
       QString result;
-      bool retcode = Widgets::passwordBox(0, title, args->getOption("password"), result);
+      const bool retcode = Widgets::passwordBox(0, title, args->getOption("password"), result);
       cout << qPrintable(result) << endl;
       return retcode ? 0 : 1;
     }
@@ -328,8 +329,8 @@ static int directCommand(KCmdLineArgs *args)
                                                    duration );
     KDialog::centerOnScreen( popup );
     QTimer *timer = new QTimer();
-    QObject::connect( timer, SIGNAL( timeout() ), kapp, SLOT( quit() ) );
-    QObject::connect( popup, SIGNAL( clicked() ), kapp, SLOT( quit() ) );
+    QObject::connect( timer, SIGNAL( timeout() ), qApp, SLOT( quit() ) );
+    QObject::connect( popup, SIGNAL( clicked() ), qApp, SLOT( quit() ) );
     timer->setSingleShot( true );
     timer->start( duration );
 
@@ -349,7 +350,7 @@ static int directCommand(KCmdLineArgs *args)
 	    popup->setAnchor( QPoint(x, y) );
 	}
 #endif
-	kapp->exec();
+	qApp->exec();
 	return 0;
       }
 
@@ -392,19 +393,19 @@ static int directCommand(KCmdLineArgs *args)
       return ret;
     }
 
-    // --combobox <text> [tag item] [tag item] ..."
+    // --combobox <text> item [item] ..."
     if (args->isSet("combobox")) {
         QStringList list;
-        if (args->count() >= 2) {
+        if (args->count() >= 1) {
             for (int i = 0; i < args->count(); i++) {
                 list.append(args->arg(i));
             }
-            QString text = args->getOption("combobox");
+            const QString text = args->getOption("combobox");
 	    if (args->isSet("default")) {
 	        defaultEntry = args->getOption("default");
 	    }
             QString result;
-	    bool retcode = Widgets::comboBox(0, title, text, list, defaultEntry, result);
+	    const bool retcode = Widgets::comboBox(0, title, text, list, defaultEntry, result);
             cout << result.toLocal8Bit().data() << endl;
             return retcode ? 0 : 1;
         }
@@ -418,12 +419,12 @@ static int directCommand(KCmdLineArgs *args)
             for (int i = 0; i < args->count(); i++) {
                 list.append(args->arg(i));
             }
-            QString text = args->getOption("menu");
+            const QString text = args->getOption("menu");
 	    if (args->isSet("default")) {
 	        defaultEntry = args->getOption("default");
 	    }
             QString result;
-            bool retcode = Widgets::listBox(0, title, text, list, defaultEntry, result);
+            const bool retcode = Widgets::listBox(0, title, text, list, defaultEntry, result);
             if (1 == retcode) { // OK was selected
 	        cout << result.toLocal8Bit().data() << endl;
 	    }
@@ -440,10 +441,10 @@ static int directCommand(KCmdLineArgs *args)
                 list.append(args->arg(i));
             }
 
-            QString text = args->getOption("checklist");
+            const QString text = args->getOption("checklist");
             QStringList result;
 
-            bool retcode = Widgets::checkList(0, title, text, list, separateOutput, result);
+            const bool retcode = Widgets::checkList(0, title, text, list, separateOutput, result);
 
             for (int i=0; i<result.count(); i++)
                 if (!result.at(i).toLocal8Bit().isEmpty()) {
@@ -462,9 +463,9 @@ static int directCommand(KCmdLineArgs *args)
                 list.append(args->arg(i));
             }
 
-            QString text = args->getOption("radiolist");
+            const QString text = args->getOption("radiolist");
             QString result;
-            bool retcode = Widgets::radioBox(0, title, text, list, result);
+            const bool retcode = Widgets::radioBox(0, title, text, list, result);
             cout << result.toLocal8Bit().data() << endl;
             exit( retcode ? 0 : 1 );
         }
@@ -489,7 +490,7 @@ static int directCommand(KCmdLineArgs *args)
 	}
 	Widgets::handleXGeometry(&dlg);
 	kapp->setTopWidget( &dlg );
-	dlg.setCaption(title.isNull() ? i18n("Open") : title);
+	dlg.setCaption(title.isEmpty() ? i18n("Open") : title);
 	dlg.exec();
 
         if (args->isSet("multiple")) {
@@ -530,7 +531,7 @@ static int directCommand(KCmdLineArgs *args)
 	dlg.setOperationMode( KFileDialog::Saving );
 	Widgets::handleXGeometry(&dlg);
 	kapp->setTopWidget( &dlg );
-	dlg.setCaption(title.isNull() ? i18n("Save As") : title);
+	dlg.setCaption(title.isEmpty() ? i18n("Save As") : title);
 	dlg.exec();
 
 	if ( args->isSet("getsaveurl") ) {
@@ -567,7 +568,7 @@ static int directCommand(KCmdLineArgs *args)
 	kapp->setTopWidget( &myDialog );
 
 	Widgets::handleXGeometry(&myDialog);
-	if ( !title.isNull() )
+	if ( !title.isEmpty() )
 	    myDialog.setCaption( title );
 
 	if ( myDialog.exec() == QDialog::Accepted )
@@ -601,7 +602,7 @@ static int directCommand(KCmdLineArgs *args)
 	}
 	Widgets::handleXGeometry(&dlg);
 	kapp->setTopWidget( &dlg );
-	dlg.setCaption(title.isNull() ? i18n("Open") : title);
+	dlg.setCaption(title.isEmpty() ? i18n("Open") : title);
 	dlg.exec();
 
         if (args->isSet("multiple")) {
@@ -627,36 +628,42 @@ static int directCommand(KCmdLineArgs *args)
         if (args->count() >= 1)  {
             contextStr = args->arg(0);
         }
-        KIconLoader::Group group = KIconLoader::NoGroup;
-        if ( groupStr == QLatin1String( "Desktop" ) )
-            group = KIconLoader::Desktop;
-        else if ( groupStr == QLatin1String( "Toolbar" ) )
-            group = KIconLoader::Toolbar;
-        else if ( groupStr == QLatin1String( "MainToolbar" ) )
-            group = KIconLoader::MainToolbar;
-        else if ( groupStr == QLatin1String( "Small" ) )
-            group = KIconLoader::Small;
-        else if ( groupStr == QLatin1String( "Panel" ) )
-            group = KIconLoader::Panel;
-        else if ( groupStr == QLatin1String( "User" ) )
-            group = KIconLoader::User;
-        KIconLoader::Context context = KIconLoader::Any;
-        // From kicontheme.cpp
-        if ( contextStr == QLatin1String( "Devices" ) )
-            context = KIconLoader::Device;
-        else if ( contextStr == QLatin1String( "MimeTypes" ) )
-            context = KIconLoader::MimeType;
-        else if ( contextStr == QLatin1String( "FileSystems" ) )
-            context = KIconLoader::FileSystem;
-        else if ( contextStr == QLatin1String( "Applications" ) )
-            context = KIconLoader::Application;
-        else if ( contextStr == QLatin1String( "Actions" ) )
-            context = KIconLoader::Action;
+        const KIconLoader::Group group =
+            ( groupStr == QLatin1String( "Desktop" ) ) ?     KIconLoader::Desktop :
+            ( groupStr == QLatin1String( "Toolbar" ) ) ?     KIconLoader::Toolbar :
+            ( groupStr == QLatin1String( "MainToolbar" ) ) ? KIconLoader::MainToolbar :
+            ( groupStr == QLatin1String( "Small" ) ) ?       KIconLoader::Small :
+            ( groupStr == QLatin1String( "Panel" ) ) ?       KIconLoader::Panel :
+            ( groupStr == QLatin1String( "Dialog" ) ) ?      KIconLoader::Dialog :
+            ( groupStr == QLatin1String( "User" ) ) ?        KIconLoader::User :
+            /* else */                                       KIconLoader::NoGroup;
+
+        const KIconLoader::Context context =
+            ( contextStr == QLatin1String( "Action" ) ) ?        KIconLoader::Action :
+            ( contextStr == QLatin1String( "Application" ) ) ?   KIconLoader::Application :
+            ( contextStr == QLatin1String( "Device" ) ) ?        KIconLoader::Device :
+            ( contextStr == QLatin1String( "FileSystem" ) ) ?    KIconLoader::FileSystem :
+            ( contextStr == QLatin1String( "MimeType" ) ) ?      KIconLoader::MimeType :
+            ( contextStr == QLatin1String( "Animation" ) ) ?     KIconLoader::Animation :
+            ( contextStr == QLatin1String( "Category" ) ) ?      KIconLoader::Category :
+            ( contextStr == QLatin1String( "Emblem" ) ) ?        KIconLoader::Emblem :
+            ( contextStr == QLatin1String( "Emote" ) ) ?         KIconLoader::Emote :
+            ( contextStr == QLatin1String( "International" ) ) ? KIconLoader::International :
+            ( contextStr == QLatin1String( "Place" ) ) ?         KIconLoader::Place :
+            ( contextStr == QLatin1String( "StatusIcon" ) ) ?    KIconLoader::StatusIcon :
+            // begin: KDE3 compatibility (useful?)
+            ( contextStr == QLatin1String( "Devices" ) ) ?       KIconLoader::Device :
+            ( contextStr == QLatin1String( "MimeTypes" ) ) ?     KIconLoader::MimeType :
+            ( contextStr == QLatin1String( "FileSystems" ) ) ?   KIconLoader::FileSystem :
+            ( contextStr == QLatin1String( "Applications" ) ) ?  KIconLoader::Application :
+            ( contextStr == QLatin1String( "Actions" ) ) ?       KIconLoader::Action :
+            // end: KDE3 compatibility
+            /* else */                                           KIconLoader::Any;
 
 	KIconDialog dlg((QWidget*)0L);
 	kapp->setTopWidget( &dlg );
 	dlg.setup( group, context);
-	if (!title.isNull())
+	if (!title.isEmpty())
 	    dlg.setCaption(title);
 	Widgets::handleXGeometry(&dlg);
 
@@ -678,7 +685,7 @@ static int directCommand(KCmdLineArgs *args)
        close(1);
 
        int totalsteps = 100;
-       QString text = args->getOption("progressbar");
+       const QString text = args->getOption("progressbar");
 
        if (args->count() == 1)
            totalsteps = args->arg(0).toInt();
@@ -696,7 +703,7 @@ static int directCommand(KCmdLineArgs *args)
         }
         Widgets::handleXGeometry(&dlg);
         kapp->setTopWidget(&dlg);
-        dlg.setCaption(title.isNull() ? i18n("Choose Color") : title);
+        dlg.setCaption(title.isEmpty() ? i18n("Choose Color") : title);
 
         if (dlg.exec() == KColorDialog::Accepted) {
             QString result;
@@ -710,6 +717,35 @@ static int directCommand(KCmdLineArgs *args)
         }
         return 1; // cancelled
     }
+    if (args->isSet("slider"))
+    {
+       int miniValue = 0;
+       int maxValue = 0;
+       int step = 0;
+       const QString text = args->getOption( "slider" );
+       if ( args->count() == 3 )
+       {
+           miniValue = args->arg(0).toInt();
+           maxValue = args->arg( 1 ).toInt();
+           step = args->arg( 2 ).toInt();
+       }
+       int result = 0;
+
+       const bool returnCode = Widgets::slider(0, title, text, miniValue, maxValue, step, result);
+       if ( returnCode )
+           cout << result << endl;
+       return returnCode;
+    }
+    if (args->isSet("calendar"))
+    {
+       const QString text = args->getOption( "calendar" );
+       QDate result;
+
+       const bool returnCode = Widgets::calendar(0, title, text, result);
+       if ( returnCode )
+           cout << result.toString().toLocal8Bit().data() << endl;
+       return returnCode;
+    }
 
     KCmdLineArgs::usage();
     return -2; // NOTREACHED
@@ -747,7 +783,7 @@ int main(int argc, char *argv[])
   options.add("password <text>", ki18n("Password dialog"));
   options.add("textbox <file> [width] [height]", ki18n("Text Box dialog"));
   options.add("textinputbox <text> <init> [width] [height]", ki18n("Text Input Box dialog"));
-  options.add("combobox <text> [tag item] [tag item] ...", ki18n("ComboBox dialog"));
+  options.add("combobox <text> item [item] [item] ...", ki18n("ComboBox dialog"));
   options.add("menu <text> [tag item] [tag item] ...", ki18n("Menu dialog"));
   options.add("checklist <text> [tag item status] ...", ki18n("Check List dialog"));
   options.add("radiolist <text> [tag item status] ...", ki18n("Radio List dialog"));
@@ -767,7 +803,8 @@ int main(int argc, char *argv[])
   options.add("separate-output", ki18n("Return list items on separate lines (for checklist option and file open with --multiple)"));
   options.add("print-winid", ki18n("Outputs the winId of each dialog"));
   options.add("dontagain <file:entry>", ki18n("Config file and option name for saving the \"do-not-show/ask-again\" state"));
-
+  options.add( "slider <text> [minvalue] [maxvalue] [step]", ki18n( "Slider dialog box, returns selected value" ) );
+  options.add( "calendar <text>", ki18n( "Calendar dialog box, returns selected date" ) );
   /* kdialog originally used --embed for attaching the dialog box.  However this is misleading and so we changed to --attach.
      * For backwards compatibility, we silently map --embed to --attach */
   options.add("attach <winid>", ki18n("Makes the dialog transient for an X app specified by winid"));
