@@ -38,7 +38,6 @@
 #include "konqmisc.h"
 #include "konqsettingsxt.h"
 #include "konqframevisitor.h"
-#include "konqproxystyle.h"
 
 #include <kacceleratormanager.h>
 #include <konqpixmapprovider.h>
@@ -47,76 +46,6 @@
 
 #include <QtGui/QStyle>
 
-//###################################################################
-
-class KonqTabsStyle : public KonqProxyStyle
-{
-public:
-  KonqTabsStyle(QWidget *parent) : KonqProxyStyle(parent) {}
-
-  void drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const;
-  QRect subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const;
-};
-
-void KonqTabsStyle::drawPrimitive(PrimitiveElement element, const QStyleOption *option,
-                                  QPainter *painter, const QWidget *widget) const
-{
-  if (element == PE_FrameTabWidget)
-  {
-    const KonqFrameTabs *tw = static_cast<const KonqFrameTabs*>(widget);
-    const QTabBar *tb = tw->tabBar();
-
-    QStyleOptionTabV2 tab;
-    tab.initFrom(tb);
-    tab.shape = tb->shape();
-    int overlap = style()->pixelMetric(PM_TabBarBaseOverlap, &tab, tb);
-
-    if (overlap <= 0 || tw->isTabBarHidden())
-      return;
-
-    QStyleOptionTabBarBase opt;
-    opt.initFrom(tb);
-
-    opt.selectedTabRect = tb->tabRect(tb->currentIndex());
-    opt.selectedTabRect = QRect(tb->mapToParent(opt.selectedTabRect.topLeft()), opt.selectedTabRect.size());
-    opt.tabBarRect = QRect(tb->mapToParent(tb->rect().topLeft()), tb->size());
-
-    if (tw->tabPosition() == QTabWidget::North)
-      opt.rect = QRect(option->rect.left(), option->rect.top(), option->rect.width(), overlap);
-    else
-      opt.rect = QRect(option->rect.left(), option->rect.bottom() - overlap - 1, option->rect.width(), overlap);
-
-    style()->drawPrimitive(PE_FrameTabBarBase, &opt, painter, tb);
-    return;
-  }
-
-  KonqProxyStyle::drawPrimitive(element, option, painter, widget);
-}
-
-QRect KonqTabsStyle::subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
-{
-  if (element == SE_TabWidgetTabContents)
-  {
-    QRect rect = style()->subElementRect(SE_TabWidgetTabPane, option, widget);
-
-    const KonqFrameTabs *tw = static_cast<const KonqFrameTabs*>(widget);
-    const QTabBar *tb = tw->tabBar();
-
-    QStyleOptionTabV2 tab;
-    tab.initFrom(tb);
-    tab.shape = tb->shape();
-    int overlap = style()->pixelMetric(PM_TabBarBaseOverlap, &tab, tw->tabBar());
-
-    if (overlap <= 0 || tw->isTabBarHidden())
-      return rect;
-
-    return tw->tabPosition() == QTabWidget::North ?
-        rect.adjusted(0, overlap, 0, 0) : rect.adjusted(0, 0, 0, -overlap);
-  }
-
-  return KonqProxyStyle::subElementRect(element, option, widget);
-}
-
 
 //###################################################################
 
@@ -126,43 +55,34 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
   : KTabWidget(parent),
     m_pPopupMenu(0),
     m_pSubPopupMenuTab(0),
-    m_rightWidget(0), m_leftWidget(0), m_alwaysTabBar(false),
-    m_konqTabsStyle( new KonqTabsStyle( this ) )
+    m_rightWidget(0), m_leftWidget(0), m_alwaysTabBar(false)
 {
   // Set an object name so the widget style can identify this widget.
   setObjectName("kde_konq_tabwidget");
+  setDocumentMode(true);
 
   KAcceleratorManager::setNoAccel(this);
 
-  // Set the widget style to a forwarding proxy style that removes the tabwidget frame,
-  // and draws a tabbar base underneath the tabbar.
-  setStyle(m_konqTabsStyle);
-
-  // The base will be drawn on the frame instead of on the tabbar, so it extends across
-  // the whole widget.
-  tabBar()->setDrawBase(false);
-
   tabBar()->setWhatsThis(i18n( "This bar contains the list of currently open tabs. Click on a tab to make it "
 			  "active. You can also use keyboard shortcuts to "
 			  "navigate through tabs. The text on the tab shows the content "
 			  "currently open in it; place your mouse over the tab to see the full title, in "
 			  "case it has been shortened to fit the tab width." ) );
-  //kDebug(1202) << "KonqFrameTabs::KonqFrameTabs()";
+  //kDebug() << "KonqFrameTabs::KonqFrameTabs()";
 
   m_pParentContainer = parentContainer;
   m_pActiveChild = 0L;
   m_pViewManager = viewManager;
 
-  connect( this, SIGNAL( currentChanged ( int ) ),
-           this, SLOT( slotCurrentChanged( int ) ) );
-
   m_MouseMiddleClickClosesTab = KonqSettings::mouseMiddleClickClosesTab();
 
   m_permanentCloseButtons = KonqSettings::permanentCloseButton();
   if (m_permanentCloseButtons) {
-    setCloseButtonEnabled( true );
+    setTabsClosable( true );
   }
-  setTabCloseActivatePrevious( KonqSettings::tabCloseActivatePrevious() );
+  tabBar()->setSelectionBehaviorOnRemove(
+      KonqSettings::tabCloseActivatePrevious() ? QTabBar::SelectPreviousTab : QTabBar::SelectRightTab );
+
   if (KonqSettings::tabPosition()=="Bottom")
     setTabPosition( QTabWidget::South );
   connect( this, SIGNAL( closeRequest( QWidget * )), SLOT(slotCloseRequest( QWidget * )));
@@ -170,9 +90,13 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
            m_pViewManager->mainWindow(), SLOT( slotRemoveTabPopup() ) );
 
   if ( KonqSettings::addTabButton() ) {
-    m_leftWidget = new QToolButton( this );
+    m_leftWidget = new NewTabToolButton( this );
     connect( m_leftWidget, SIGNAL( clicked() ),
              m_pViewManager->mainWindow(), SLOT( slotAddTab() ) );
+    connect( m_leftWidget, SIGNAL( testCanDecode(const QDragMoveEvent *, bool & )),
+             SLOT( slotTestCanDecode(const QDragMoveEvent *, bool & ) ) );
+    connect( m_leftWidget, SIGNAL( receivedDropEvent( QDropEvent * )),
+             SLOT( slotReceivedDropEvent( QDropEvent * ) ) );
     m_leftWidget->setIcon( KIcon( "tab-new" ) );
     m_leftWidget->adjustSize();
     m_leftWidget->setToolTip( i18n("Open a new tab"));
@@ -214,10 +138,9 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
 
 KonqFrameTabs::~KonqFrameTabs()
 {
-  //kDebug(1202) << "KonqFrameTabs::~KonqFrameTabs() " << this << " - " << className();
+  //kDebug() << "KonqFrameTabs::~KonqFrameTabs() " << this << " - " << className();
   qDeleteAll( m_childFrameList );
   m_childFrameList.clear();
-  delete m_konqTabsStyle;
 }
 
 void KonqFrameTabs::saveConfig( KConfigGroup& config, const QString &prefix, const KonqFrameBase::Options &options,
@@ -246,12 +169,12 @@ void KonqFrameTabs::copyHistory( KonqFrameBase *other )
 {
 
   if( !other ) {
-    kDebug(1202) << "The Frame does not exist";
+    kDebug() << "The Frame does not exist";
     return;
   }
 
   if(other->frameType() != KonqFrameBase::Tabs) {
-    kDebug(1202) << "Frame types are not the same";
+    kDebug() << "Frame types are not the same";
     return;
   }
 
@@ -263,7 +186,7 @@ void KonqFrameTabs::copyHistory( KonqFrameBase *other )
 
 void KonqFrameTabs::setTitle( const QString &title , QWidget* sender)
 {
-  // kDebug(1202) << "KonqFrameTabs::setTitle( " << title << " , " << sender << " )";
+  // kDebug() << "KonqFrameTabs::setTitle( " << title << " , " << sender << " )";
   // Make sure that '&' is displayed correctly
   QString tabText( title );
   setTabText( indexOf( sender ), tabText.replace('&', "&&") );
@@ -271,7 +194,7 @@ void KonqFrameTabs::setTitle( const QString &title , QWidget* sender)
 
 void KonqFrameTabs::setTabIcon( const KUrl &url, QWidget* sender )
 {
-  //kDebug(1202) << "KonqFrameTabs::setTabIcon( " << url << " , " << sender << " )";
+  //kDebug() << "KonqFrameTabs::setTabIcon( " << url << " , " << sender << " )";
   KIcon iconSet = KIcon( KonqPixmapProvider::self()->iconNameFor( url ) );
   const int pos = indexOf(sender);
   if (tabIcon(pos).pixmap(iconSize()).serialNumber() != iconSet.pixmap(iconSize()).serialNumber())
@@ -288,14 +211,14 @@ void KonqFrameTabs::activateChild()
 
 void KonqFrameTabs::insertChildFrame( KonqFrameBase* frame, int index )
 {
-    //kDebug(1202) << "KonqFrameTabs " << this << ": insertChildFrame " << frame;
+    //kDebug() << "KonqFrameTabs " << this << ": insertChildFrame " << frame;
 
     if (!frame) {
-        kWarning(1202) << "KonqFrameTabs " << this << ": insertChildFrame(0) !" ;
+        kWarning() << "KonqFrameTabs " << this << ": insertChildFrame(0) !" ;
         return;
     }
 
-    //kDebug(1202) << "Adding frame";
+    //kDebug() << "Adding frame";
 
     //QTabWidget docs say that inserting tabs while already shown causes
     //flicker...
@@ -311,6 +234,13 @@ void KonqFrameTabs::insertChildFrame( KonqFrameBase* frame, int index )
     // note that this can call slotCurrentChanged (e.g. when inserting/replacing the first tab)
     insertTab(index, frame->asQWidget(), "");
 
+    // Connect to currentChanged only after inserting the first tab,
+    // otherwise insertTab() can call slotCurrentChanged, which we don't expect
+    // (the part isn't in the partmanager yet; better let konqviewmanager take care
+    // of setting the active part)
+    connect(this, SIGNAL(currentChanged(int)),
+            this, SLOT(slotCurrentChanged(int)), Qt::UniqueConnection);
+
     if (m_rightWidget) {
       m_rightWidget->setEnabled( m_childFrameList.count() > 1 );
     }
@@ -326,7 +256,7 @@ void KonqFrameTabs::insertChildFrame( KonqFrameBase* frame, int index )
 
 void KonqFrameTabs::childFrameRemoved( KonqFrameBase * frame )
 {
-  //kDebug(1202) << "KonqFrameTabs::RemoveChildFrame " << this << ". Child " << frame << " removed";
+  //kDebug() << "KonqFrameTabs::RemoveChildFrame " << this << ". Child " << frame << " removed";
   if (frame) {
     removeTab(indexOf(frame->asQWidget()));
     m_childFrameList.removeAll(frame);
@@ -336,9 +266,9 @@ void KonqFrameTabs::childFrameRemoved( KonqFrameBase * frame )
       updateTabBarVisibility();
   }
   else
-    kWarning(1202) << "KonqFrameTabs " << this << ": childFrameRemoved(0L) !" ;
+    kWarning() << "KonqFrameTabs " << this << ": childFrameRemoved(0L) !" ;
 
-  //kDebug(1202) << "KonqFrameTabs::RemoveChildFrame finished";
+  //kDebug() << "KonqFrameTabs::RemoveChildFrame finished";
 }
 
 void KonqFrameTabs::moveTabBackward( int index )
@@ -392,11 +322,7 @@ void KonqFrameTabs::slotContextMenu( QWidget *w, const QPoint &p )
   m_popupActions["othertabs"]->setEnabled( true );
   m_popupActions["closeothertabs"]->setEnabled( true );
 
-  // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a
-  // class hierarchy and it could crash one day, but I haven't checked
-  // setWorkingTab so I don't know if it can handle nulls.
-
-  m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
+  m_pViewManager->mainWindow()->setWorkingTab( indexOf(w) );
   m_pPopupMenu->exec( p );
 }
 
@@ -437,8 +363,7 @@ void KonqFrameTabs::refreshSubPopupMenuTab()
 void KonqFrameTabs::slotCloseRequest( QWidget *w )
 {
   if ( m_childFrameList.count() > 1 ) {
-    // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a class hierarchy and it could crash one day, but I haven't checked setWorkingTab so I don't know if it can handle nulls.
-    m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
+    m_pViewManager->mainWindow()->setWorkingTab( indexOf(w) );
     emit removeTabPopup();
   }
 }
@@ -463,11 +388,7 @@ void KonqFrameTabs::slotMouseMiddleClick()
 void KonqFrameTabs::slotMouseMiddleClick( QWidget *w )
 {
   if ( m_MouseMiddleClickClosesTab ) {
-    if ( m_childFrameList.count() > 1 ) {
-      // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a class hierarchy and it could crash one day, but I haven't checked setWorkingTab so I don't know if it can handle nulls.
-      m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
-      emit removeTabPopup();
-    }
+    slotCloseRequest(w);
   }
   else {
   KUrl filteredURL ( KonqMisc::konqFilteredURL( this, QApplication::clipboard()->text(QClipboard::Selection ) ) );
@@ -589,10 +510,19 @@ bool KonqFrameTabs::accept( KonqFrameVisitor* visitor )
 {
     if ( !visitor->visit( this ) )
         return false;
-    foreach( KonqFrameBase* frame, m_childFrameList ) {
-        Q_ASSERT( frame );
-        if ( !frame->accept( visitor ) )
+    if (visitor->visitAllTabs()) {
+        foreach(KonqFrameBase* frame, m_childFrameList) {
+        Q_ASSERT(frame);
+        if (!frame->accept(visitor))
             return false;
+        }
+    } else {
+        // visit only current tab
+        if (m_pActiveChild) {
+            if (!m_pActiveChild->accept(visitor)) {
+                return false;
+            }
+        }
     }
     if ( !visitor->endVisit( this ) )
         return false;
@@ -604,11 +534,13 @@ void KonqFrameTabs::slotCurrentChanged( int index )
     const KColorScheme colorScheme(QPalette::Active, KColorScheme::Window);
     setTabTextColor(index, colorScheme.foreground(KColorScheme::NormalText).color());
 
-    KonqFrameBase* currentFrame = dynamic_cast<KonqFrameBase*>(widget( index ));
+    KonqFrameBase* currentFrame = tabAt(index);
     if (currentFrame && !m_pViewManager->isLoadingProfile()) {
         m_pActiveChild = currentFrame;
         currentFrame->activateChild();
     }
+
+    m_pViewManager->mainWindow()->linkableViewCountChanged();
 }
 
 /**
@@ -669,4 +601,9 @@ void KonqFrameTabs::replaceChildFrame(KonqFrameBase* oldFrame, KonqFrameBase* ne
     setCurrentIndex(index);
 }
 
+KonqFrameBase* KonqFrameTabs::tabAt(int index) const
+{
+    return dynamic_cast<KonqFrameBase*>(widget(index));
+}
+
 #include "konqtabs.moc"
