@@ -17,7 +17,9 @@
 
 #include <qtest_kde.h>
 #include "konqviewmgrtest.h"
+#include "../konqsettingsxt.h"
 #include <QToolBar>
+#include <qtestmouse.h>
 
 #include <konqframe.h>
 #include <konqmainwindow.h>
@@ -25,8 +27,10 @@
 #include <konqview.h>
 #include <konqtabs.h>
 #include <konqframevisitor.h>
+#include <konqsessionmanager.h>
 #include <kstandarddirs.h>
 #include <kconfiggroup.h>
+#include <kio/job.h>
 #include <QLayout>
 
 #include <khtml_part.h>
@@ -61,6 +65,7 @@ void ViewMgrTest::sendAllPendingResizeEvents( QWidget* mainWindow )
         allChildWidgets.prepend( mainWindow );
         foreach( QWidget* w, allChildWidgets ) {
             if (w->testAttribute(Qt::WA_PendingResizeEvent)) {
+                w->setAttribute(Qt::WA_WState_Created, true); // hack: avoid assert in Qt-4.6
                 //qDebug() << "Resizing" << w << " to " << w->size() << endl;
                 QResizeEvent e(w->size(), QSize());
                 QApplication::sendEvent(w, &e);
@@ -109,7 +114,10 @@ private:
 
 void ViewMgrTest::initTestCase()
 {
-    QVERIFY( KGlobal::mainComponent().componentName() == "konqueror" );
+    KonqSessionManager::self()->disableAutosave();
+    QCOMPARE(KGlobal::mainComponent().componentName(), QString("konqueror"));
+    QCOMPARE(KonqSettings::mmbOpensTab(), true);
+    QCOMPARE(KonqSettings::popupsWithinTabs(), false);
 }
 
 void ViewMgrTest::testCreateFirstView()
@@ -122,6 +130,7 @@ void ViewMgrTest::testCreateFirstView()
 
     // Use DebugFrameVisitor to find out the structure of the frame hierarchy
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[F].") ); // mainWindow, one tab, one frame
+    QCOMPARE(mainWindow.linkableViewsCount(), 1);
 
     // Check widget parents:  part's widget -> frame -> [tabwidget's stack] -> tabwidget -> mainWindow
     QWidget* partWidget = view->part()->widget();
@@ -144,6 +153,7 @@ void ViewMgrTest::testCreateFirstView()
     //    if (!toolbar->isHidden())
     //        qDebug() << toolbar << toolbar->geometry();
     //}
+    //qDebug() << "frame geom: " << frame->geometry();
     QVERIFY( frame->width() > 680 );
     QVERIFY( frame->height() > 240 ); // usually 325, but can be 256 with oxygen when three toolbars are shown
     //qDebug() << "partWidget geom:" << partWidget->geometry();
@@ -151,7 +161,7 @@ void ViewMgrTest::testCreateFirstView()
     QVERIFY( partWidget->height() > frame->height() - 50 /*statusbar*/ );
     //qDebug() << "tabWidget geom: " << tabWidget->geometry();
     QVERIFY( tabWidget->width() > 680 );
-    QVERIFY( tabWidget->height() > frame->height() );
+    QVERIFY( tabWidget->height() >= frame->height() ); // equal, unless there's a border in the frame
 }
 
 void ViewMgrTest::testRemoveFirstView()
@@ -176,6 +186,8 @@ void ViewMgrTest::testSplitView()
     QVERIFY( view2 );
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(FF)].") ); // mainWindow, tab widget, one splitter, two frames
 
+    QCOMPARE(mainWindow.linkableViewsCount(), 2);
+
     // Check widget parents
     //mainWindow.dumpObjectTree();
     QWidget* partWidget = view->part()->widget();
@@ -227,17 +239,25 @@ void ViewMgrTest::testSplitView()
 
 
     // Split again
+    int widthFrame2 = frame2->width();
     KonqView* view3 = viewManager->splitView( view, Qt::Horizontal );
     QVERIFY( view3 );
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(C(FF)F)].") );
+    // Check that the width of the second frame has not changed (bug 160407)
+    QCOMPARE( frame2->width(), widthFrame2 );
+    QCOMPARE(mainWindow.linkableViewsCount(), 3);
 
     // Now test removing the first view
     viewManager->removeView( view );
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(FF)].") ); // mainWindow, tab widget, one splitter, two frames
+    // Check again that the width of the second frame has not changed (bug 160407 comments 18-20)
+    QCOMPARE( frame2->width(), widthFrame2 );
+    QCOMPARE(mainWindow.linkableViewsCount(), 2);
 
     // Now test removing the last view
     viewManager->removeView( view3 );
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[F].") ); // mainWindow, tab widget, one frame
+    QCOMPARE(mainWindow.linkableViewsCount(), 1);
 }
 
 void ViewMgrTest::testSplitMainContainer()
@@ -281,16 +301,21 @@ void ViewMgrTest::testSplitMainContainer()
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[F].") ); // mainWindow, tab widget, one frame
 }
 
-void ViewMgrTest::testLinkedViews()
+static void openHtmlWithLink(KonqMainWindow& mainWindow)
 {
-    // First part is much like KonqHtmlTest::loadSimpleHtml.
-    KonqMainWindow mainWindow;
-    //KonqViewManager* viewManager = mainWindow.viewManager();
+    // Much like KonqHtmlTest::loadSimpleHtml.
     mainWindow.openUrl(0, KUrl("data:text/html, <a href=\"data:text/html, Link target\">Click me</a>"), "text/html");
     KonqView* view = mainWindow.currentView();
     QVERIFY(view);
     QVERIFY(QTest::kWaitForSignal(view, SIGNAL(viewCompleted(KonqView*)), 20000));
     QCOMPARE(view->serviceType(), QString("text/html"));
+}
+
+void ViewMgrTest::testLinkedViews()
+{
+    KonqMainWindow mainWindow;
+    openHtmlWithLink(mainWindow);
+    KonqView* view = mainWindow.currentView();
     // Split it
     qDebug() << "SPLITTING";
     mainWindow.slotSplitViewHorizontal();
@@ -301,6 +326,7 @@ void ViewMgrTest::testLinkedViews()
     QVERIFY(QTest::kWaitForSignal(view2, SIGNAL(viewCompleted(KonqView*)), 20000));
     KUrl origUrl = view->url();
     QCOMPARE(view2->url().url(), origUrl.url());
+    QCOMPARE(mainWindow.linkableViewsCount(), 2);
     view->setLinkedView(true);
     view2->setLinkedView(true);
     view->setLockedLocation(true);
@@ -318,6 +344,73 @@ void ViewMgrTest::testLinkedViews()
     QCOMPARE(view2->url().url(), KUrl("data:text/html, Link target").url());
 }
 
+void ViewMgrTest::testPopupNewTab() // RMB, "Open in new tab"
+{
+    KonqMainWindow mainWindow;
+    openHtmlWithLink(mainWindow);
+    KFileItem item(KUrl("data:text/html, hello"), "text/html", S_IFREG);
+    mainWindow.prepareForPopupMenu(KFileItemList() << item, KParts::OpenUrlArguments(), KParts::BrowserArguments());
+    QMetaObject::invokeMethod(&mainWindow, "slotPopupNewTab");
+    QTest::qWait(1000);
+    QCOMPARE(DebugFrameVisitor::inspect(&mainWindow), QString("MT[FF].")); // mainWindow, tab widget, two tabs
+    QCOMPARE(KMainWindow::memberList().count(), 1);
+    QCOMPARE(mainWindow.linkableViewsCount(), 1);
+}
+
+static void checkSecondWindowHasOneTab() // and delete it.
+{
+    QCOMPARE(KMainWindow::memberList().count(), 2);
+    KonqMainWindow* newWindow = qobject_cast<KonqMainWindow*>(KMainWindow::memberList().last());
+    QVERIFY(newWindow);
+    QCOMPARE(DebugFrameVisitor::inspect(newWindow), QString("MT[F].")); // mainWindow, tab widget, one tab
+    delete newWindow;
+}
+
+void ViewMgrTest::testPopupNewWindow() // RMB, "Open new window"
+{
+    KonqMainWindow mainWindow;
+    openHtmlWithLink(mainWindow);
+    KFileItem item(KUrl("data:text/html, hello"), "text/html", S_IFREG);
+    mainWindow.prepareForPopupMenu(KFileItemList() << item, KParts::OpenUrlArguments(), KParts::BrowserArguments());
+    QMetaObject::invokeMethod(&mainWindow, "slotPopupNewWindow");
+    QTest::qWait(100);
+    QCOMPARE(DebugFrameVisitor::inspect(&mainWindow), QString("MT[F].")); // mainWindow, tab widget, one tab
+    QVERIFY(KMainWindow::memberList().last() != &mainWindow);
+    checkSecondWindowHasOneTab();
+}
+
+void ViewMgrTest::testCtrlClickOnLink()
+{
+    KonqMainWindow mainWindow;
+    openHtmlWithLink(mainWindow);
+    KonqView* view = mainWindow.currentView();
+    KHTMLPart* part = qobject_cast<KHTMLPart *>(view->part());
+    qDebug() << "CLICKING NOW";
+    QTest::mouseClick(part->view()->widget(), Qt::LeftButton, Qt::ControlModifier, QPoint(10, 10));
+    QTest::qWait(100);
+    // Expected behavior for Ctrl+click:
+    //  new tab, if mmbOpensTab
+    //  new window, if !mmbOpensTab
+    if (KonqSettings::mmbOpensTab()) {
+        QCOMPARE(DebugFrameVisitor::inspect(&mainWindow), QString("MT[FF].")); // mainWindow, tab widget, two tabs
+        QCOMPARE(KMainWindow::memberList().count(), 1);
+    } else {
+        QCOMPARE(DebugFrameVisitor::inspect(&mainWindow), QString("MT[F].")); // mainWindow, tab widget, one tab
+        checkSecondWindowHasOneTab();
+    }
+}
+
+void ViewMgrTest::sameTestsWithMmbOpenTabsFalse()
+{
+    // Redo testPopupNewTab, testPopupNewWindow and testCtrlClickOnLink,
+    // but as if the user (e.g. Pino) had disabled the setting "Open links in new tabs".
+    KonqSettings::setMmbOpensTab(false);
+    testPopupNewTab();
+    testPopupNewWindow();
+    testCtrlClickOnLink();
+    KonqSettings::setMmbOpensTab(true);
+}
+
 static void openTabWithTitle(KonqMainWindow& mainWindow, const QString& title, KonqView*& view)
 {
     KonqViewManager* viewManager = mainWindow.viewManager();
@@ -352,13 +445,14 @@ void ViewMgrTest::testAddTabs()
 
     KTabWidget* tabWidget = mainWindow.findChild<KTabWidget*>();
     QVERIFY(tabWidget);
-    KonqView* viewTab2, *viewTab3, *viewTab4, *viewTab5;
-    openTabWithTitle(mainWindow, titles[1], viewTab2);
-    openTabWithTitle(mainWindow, titles[2], viewTab3);
-    openTabWithTitle(mainWindow, titles[3], viewTab4);
-    openTabWithTitle(mainWindow, titles[4], viewTab5);
+    KonqView* viewTab1, *viewTab2, *viewTab3, *viewTab4;
+    openTabWithTitle(mainWindow, titles[1], viewTab1);
+    openTabWithTitle(mainWindow, titles[2], viewTab2);
+    openTabWithTitle(mainWindow, titles[3], viewTab3);
+    openTabWithTitle(mainWindow, titles[4], viewTab4);
     for (int i = 0; i < titles.count(); ++i)
         QCOMPARE(tabWidget->tabText(i), QString(titles[i]));
+    QPointer<KonqView> viewTab2Pointer(viewTab2);
 
     // Ensure tabwidget has a nice size
     mainWindow.resize(599, 699);
@@ -369,6 +463,7 @@ void ViewMgrTest::testAddTabs()
     KonqFrameBase* frame = dynamic_cast<KonqFrameBase*>(viewManager->tabContainer()->currentWidget());
     QVERIFY(frame);
     viewManager->removeTab(frame);
+    QVERIFY(viewTab2Pointer.isNull()); // check the view got deleted
     QList<int> expectedTitles; expectedTitles << 0 << 1 << 3 << 4;
     for (int i = 0; i < expectedTitles.count(); ++i)
         QCOMPARE(tabWidget->tabText(i), titles[expectedTitles[i]]);
@@ -407,26 +502,39 @@ void ViewMgrTest::testDuplicateSplittedTab()
 
     viewManager->duplicateTab(container); // TODO shouldn't it return a KonqFrameBase?
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(FF)C(FF)].") ); // mainWindow, tab widget, two tabs
+    QCOMPARE(mainWindow.linkableViewsCount(), 2);
 
     viewManager->removeTab(container);
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(FF)].") ); // mainWindow, tab widget, one tab
+    QCOMPARE(mainWindow.linkableViewsCount(), 2);
 }
 
 // Like in http://bugs.kde.org/show_bug.cgi?id=153533,
 // where the part deletes itself.
 void ViewMgrTest::testDeletePartInTab()
 {
-    KonqMainWindow mainWindow;
-    KonqViewManager* viewManager = mainWindow.viewManager();
-    KonqView* view = viewManager->createFirstView( "KonqAboutPage", "konq_aboutpage" );
-    QVERIFY( view );
+    QPointer<KonqMainWindow> mainWindow = new KonqMainWindow;
+    QVERIFY(mainWindow->testAttribute(Qt::WA_DeleteOnClose));
+    KonqViewManager* viewManager = mainWindow->viewManager();
+    QPointer<KonqView> view = viewManager->createFirstView( "KonqAboutPage", "konq_aboutpage" );
+    QVERIFY(!view.isNull());
+    QPointer<QWidget> partWidget = view->part()->widget();
+
     QPointer<KonqView> viewTab2 = viewManager->addTab("text/html");
+    QPointer<QWidget> partWidget2 = viewTab2->part()->widget();
     QVERIFY(!viewTab2.isNull());
-    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[FF].") ); // mainWindow, tab widget, two tabs
+    QCOMPARE( DebugFrameVisitor::inspect(mainWindow), QString("MT[FF].") ); // mainWindow, tab widget, two tabs
 
     delete viewTab2->part();
     QVERIFY(viewTab2.isNull());
-    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[F].") ); // mainWindow, tab widget, one tab
+    QVERIFY(partWidget2.isNull());
+    QCOMPARE( DebugFrameVisitor::inspect(mainWindow), QString("MT[F].") ); // mainWindow, tab widget, one tab
+
+    delete view->part();
+    QVERIFY(view.isNull());
+    QVERIFY(partWidget.isNull());
+    qApp->sendPostedEvents(0, QEvent::DeferredDelete);
+    QVERIFY(mainWindow.isNull());
 }
 
 static void loadFileManagementProfile(KonqMainWindow& mainWindow)
@@ -469,6 +577,24 @@ void ViewMgrTest::testLoadProfile()
     QVERIFY(sizes[0] < sizes[1]);
 }
 
+void ViewMgrTest::testLoadOldProfile()
+{
+    KonqMainWindow mainWindow;
+
+    const QString profileSrc = KDESRCDIR "/filemanagement.old.profile";
+    const QString profile = profileSrc + ".copy";
+    // KonqViewManager fixes up the old profile, so let's make a copy of it first.
+    KIO::FileCopyJob* job = KIO::file_copy(profileSrc, profile, -1, KIO::Overwrite);
+    QVERIFY(job->exec());
+    const QString path = QDir::homePath();
+    mainWindow.viewManager()->loadViewProfileFromFile(profile, "filemanagement", KUrl(path));
+    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MC(FT[F]).") ); // mainWindow, splitter, frame, tab widget, one frame
+    QCOMPARE(mainWindow.locationBarURL(), path);
+    QCOMPARE(mainWindow.currentView()->locationBarURL(), path);
+    QFile::remove(profile);
+}
+
+
 void ViewMgrTest::testDuplicateWindow()
 {
     KonqMainWindow mainWindow;
@@ -493,11 +619,35 @@ void ViewMgrTest::testCloseOtherTabs()
     viewManager->splitView( viewTab2, Qt::Horizontal );
     viewManager->addTab("text/html");
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[FFC(FF)F].") ); // mainWindow, tab widget, first tab = one frame, second tab = one frame, third tab = splitter with two frames, fourth tab = one frame
-    mainWindow.setWorkingTab(viewTab2->frame()); // This actually sets a frame which is not a tab, but KonqViewManager::removeOtherTabs should be able to deal with these cases
+    KTabWidget* tabWidget = mainWindow.findChild<KTabWidget*>();
+    tabWidget->setCurrentIndex(2);
+    QCOMPARE(mainWindow.linkableViewsCount(), 2);
+    tabWidget->setCurrentIndex(3);
+    QCOMPARE(mainWindow.linkableViewsCount(), 1);
+
+    mainWindow.setWorkingTab(2); // This actually sets a frame which is not a tab, but KonqViewManager::removeOtherTabs should be able to deal with these cases
     mainWindow.slotRemoveOtherTabsPopupDelayed();
     QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(FF)].") ); // mainWindow, tab widget, first tab = splitter with two frames
 }
 
+void ViewMgrTest::testCloseTabsFast() // #210551/#150162
+{
+    KonqMainWindow mainWindow;
+    mainWindow.openUrl(0, KUrl("data:text/html, <p>Hello World</p>"), "text/html");
+    KonqViewManager* viewManager = mainWindow.viewManager();
+    viewManager->addTab("text/html");
+    viewManager->addTab("text/html");
+    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[FFF].") ); // mainWindow, tab widget, 3 simple tabs
+    KTabWidget* tabWidget = mainWindow.findChild<KTabWidget*>();
+    tabWidget->setCurrentIndex(2);
+
+    mainWindow.setWorkingTab(1);
+    mainWindow.slotRemoveTabPopup();
+    mainWindow.slotRemoveTabPopup();
+    QTest::qWait(100); // process the delayed invocations
+    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[F].") ); // mainWindow, tab widget, 1 tab left
+}
+
 void ViewMgrTest::testDuplicateWindowWithSidebar()
 {
     KonqMainWindow mainWindow;
@@ -527,4 +677,59 @@ void ViewMgrTest::testBrowserArgumentsNewTab()
 //    QCOMPARE(view->url(), KUrl("data:text/html, <p>Second tab test</p>"));
 }
 
+void ViewMgrTest::testBreakOffTab()
+{
+    KonqMainWindow mainWindow;
+    KonqViewManager* viewManager = mainWindow.viewManager();
+    KonqView* view = viewManager->createFirstView( "KonqAboutPage", "konq_aboutpage" );
+
+    KonqFrameBase* tab = view->frame();
+    viewManager->duplicateTab( tab );
+    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[FF].") ); // mainWindow, tab widget, two tabs
+
+    // Break off a tab
+
+    KonqMainWindow* mainWindow2 = viewManager->breakOffTab( tab, mainWindow.size() );
+    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[F].") ); // mainWindow, one tab, one frame
+    QCOMPARE( DebugFrameVisitor::inspect(mainWindow2), QString("MT[F].") ); // mainWindow, one tab, one frame
+
+    // Verify that the new tab container has an active child and that duplicating the tab in the new window does not crash (bug 203069)
+
+    QVERIFY( mainWindow2->viewManager()->tabContainer()->activeChild() );
+    mainWindow2->viewManager()->duplicateTab( mainWindow2->activeChildView()->frame() );
+    QCOMPARE( DebugFrameVisitor::inspect(mainWindow2), QString("MT[FF].") ); // mainWindow, tab widget, two tabs
+
+    delete mainWindow2;
+
+    // Now split the remaining view, duplicate the tab and verify that breaking off a split tab does not crash (bug 174292).
+    // Also check that the tab container of the new main window has an active child.
+
+    view = mainWindow.activeChildView();
+    viewManager->splitView( view, Qt::Vertical );
+    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(FF)].") ); // mainWindow, tab widget, one splitter, two frames
+    KonqFrameContainerBase* container = view->frame()->parentContainer();
+    viewManager->duplicateTab( container );
+    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(FF)C(FF)].") ); // mainWindow, tab widget, two tabs with split views
+    mainWindow2 = viewManager->breakOffTab( container, mainWindow.size() );
+    QCOMPARE( DebugFrameVisitor::inspect(&mainWindow), QString("MT[C(FF)].") ); // mainWindow, tab widget, one splitter, two frames
+    QCOMPARE( DebugFrameVisitor::inspect(mainWindow2), QString("MT[C(FF)].") ); // mainWindow, tab widget, one splitter, two frames
+    QVERIFY( mainWindow2->viewManager()->tabContainer()->activeChild() );
+
+    delete mainWindow2;
+
+    // Verify that breaking off a tab preserves the view profile (bug 210686)
+
+    const QString profile = KStandardDirs::locate("data", "konqueror/profiles/webbrowsing");
+    QVERIFY(!profile.isEmpty());
+    const QString path = QDir::homePath();
+    mainWindow.viewManager()->loadViewProfileFromFile(profile, "webbrowsing");
+    view = mainWindow.activeChildView();
+    tab = view->frame();
+    viewManager->duplicateTab( tab );
+    mainWindow2 = viewManager->breakOffTab( tab, mainWindow.size() );
+    QCOMPARE( viewManager->currentProfile(), mainWindow2->viewManager()->currentProfile() );
+
+    delete mainWindow2;
+}
+
 #include "konqviewmgrtest.moc"
