@@ -34,7 +34,9 @@
 // KDE
 #include <KSharedPtr>
 #include <KDebug>
-#include <kdemacros.h>
+
+// Konsole
+#include "konsole_export.h"
 
 class KConfig;
 class KConfigGroup;
@@ -61,7 +63,7 @@ class ProfileGroup;
  * Profiles can be loaded from disk using ProfileReader instances
  * and saved to disk using ProfileWriter instances.
  */
-class KDE_EXPORT Profile : public QSharedData 
+class KONSOLEPRIVATE_EXPORT Profile : public QSharedData 
 {
 
 friend class KDE4ProfileReader;
@@ -172,6 +174,10 @@ public:
         WordCharacters,
         /** (TabBarPositionEnum) Position of the tab-bar relative to the terminal displays. */
         TabBarPosition,
+        /** (bool) If true, the triple click selects from current word onwards. Otherwise
+         * selects whole line.
+         */
+        TripleClickMode,
         /** (String) Default text codec */
         DefaultEncoding,
         /** (bool) Whether fonts should be aliased or not */
@@ -180,7 +186,11 @@ public:
          * currently active session. */
         StartInCurrentSessionDir,
         /** (bool) Whether a 'New Tab' and 'Close Tab' buttons should be shown on the tab bar */
-        ShowNewAndCloseTabButtons
+        ShowNewAndCloseTabButtons,
+        /** Index of profile in the File Menu
+         * In future, format will be #.#.# to account for levels
+         */
+        MenuIndex
     };
 
     /** 
@@ -352,6 +362,11 @@ public:
     /** Convenience method for property<QStringList>(Profile::Environment) */
     QStringList environment() const { return property<QStringList>(Profile::Environment); }
 
+    /** Convenience method for property<QString>(Profile::MenuIndex) */
+    QString menuIndex() const { return property<QString>(Profile::MenuIndex); }
+
+    int menuIndexAsInt() const;
+
     /**
      * Returns true if @p name has been associated with an element
      * from the Property enum or false otherwise.
@@ -362,7 +377,7 @@ public:
      * Returns the element from the Property enum associated with the 
      * specified @p name.
      *
-     * @param The name of the property to look for, this is case insensitive.
+     * @param name The name of the property to look for, this is case insensitive.
      */
     static Property lookupByName(const QString& name);
     /**
@@ -439,7 +454,7 @@ inline bool Profile::canInheritProperty(Property property)
  * This can be used as a parent for other profiles or a fallback in case
  * a profile cannot be loaded from disk.
  */
-class FallbackProfile : public Profile
+class KONSOLEPRIVATE_EXPORT FallbackProfile : public Profile
 {
 public:
     FallbackProfile();
@@ -458,7 +473,7 @@ public:
  * The Profile::Name and Profile::Path properties are unique to individual profiles,
  * setting these properties on a ProfileGroup has no effect.
  */
-class KDE_EXPORT ProfileGroup : public Profile
+class KONSOLEPRIVATE_EXPORT ProfileGroup : public Profile
 {
 public:
     typedef KSharedPtr<ProfileGroup> Ptr;
@@ -598,7 +613,7 @@ private:
  *   Icon=konsole;Directory=/home/bob
  * @endcode
  */
-class ProfileCommandParser
+class KONSOLEPRIVATE_EXPORT ProfileCommandParser
 {
 public:
     /**
