@@ -1,5 +1,6 @@
 /*
     Copyright 2006-2008 by Robert Knight <robertknight@gmail.com>
+    Copyright 2009 by Thomas Dreibholz <dreibh@iem.uni-due.de>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,10 +32,13 @@
 #include <KInputDialog>
 #include <KLocale>
 #include <KMenu>
+#include <KMessageBox>
 #include <KRun>
 #include <kshell.h>
+#include <KStandardDirs>
 #include <KToggleAction>
 #include <KUrl>
+#include <KXmlGuiWindow>
 #include <KXMLGUIFactory>
 #include <KXMLGUIBuilder>
 #include <kdebug.h>
@@ -95,12 +99,10 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     Q_ASSERT( view );
 
     // handle user interface related to session (menus etc.)
-
-#ifdef KONSOLE_PART
-    setXMLFile("konsole/partui.rc");
-#else
-    setXMLFile("konsole/sessionui.rc");
-#endif
+    if (isKonsolePart())
+        setXMLFile("konsole/partui.rc");
+    else
+        setXMLFile("konsole/sessionui.rc");
 
     setupActions();
     actionCollection()->addAssociatedWidget(view);
@@ -140,7 +142,10 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     // listen for output changes to set activity flag
     connect( _session->emulation() , SIGNAL(outputChanged()) , this ,
             SLOT(fireActivity()) );
-    
+
+    // listen for detection of ZModem transfer
+    connect( _session , SIGNAL(zmodemDetected()) , this , SLOT(zmodemDownload()) ); 
+
     // listen for flow control status changes
     connect( _session , SIGNAL(flowControlEnabledChanged(bool)) , _view ,
         SLOT(setFlowControlWarningEnabled(bool)) );
@@ -206,9 +211,11 @@ void SessionController::snapshot()
     QString title = _session->getDynamicTitle();    
     title         = title.simplified();
 
-    // crude indicator when the session is broadcasting to others
-    if (_copyToGroup && _copyToGroup->sessions().count() > 1)
+    // Visualize that the session is broadcasting to others
+    if (_copyToGroup && _copyToGroup->sessions().count() > 1) {
         title.append('*');
+    }
+    updateSessionIcon();
 
     // apply new title
     if ( !title.isEmpty() )
@@ -245,7 +252,7 @@ void SessionController::openUrl( const KUrl& url )
         _session->emulation()->sendText("ssh ");
 
         if ( url.port() > -1 )
-            _session->emulation()->sendText("-p " + QString::number(url.port()) + ' ');
+            _session->emulation()->sendText("-p " + QString::number(url.port()) + ' ' );
         if ( url.hasUser() )
             _session->emulation()->sendText(url.user() + '@');
         if ( url.hasHost() )
@@ -266,6 +273,11 @@ void SessionController::openUrl( const KUrl& url )
     else
     {
         //TODO Implement handling for other Url types
+
+        KMessageBox::sorry(_view->window(),
+                           i18n("Konsole does not know how to open the bookmark: ") +
+                           url.prettyUrl());
+
         kWarning(1211) << "Unable to open bookmark at url" << url << ", I do not know"
            << " how to handle the protocol " << url.protocol();
     }
@@ -289,6 +301,12 @@ bool SessionController::eventFilter(QObject* watched , QEvent* event)
             // second, connect the newly focused view to listen for the session's bell signal
             connect( _session , SIGNAL(bellRequest(const QString&)) ,
                     _view , SLOT(bell(const QString&)) );
+                    
+            if(_copyToAllTabsAction->isChecked()) {
+                // A session with "Copy To All Tabs" has come into focus:
+                // Ensure that newly created sessions are included in _copyToGroup!
+                copyInputToAllTabs();
+            }
         }
         // when a mouse move is received, create the URL filter and listen for output changes if
         // it has not already been created.  If it already exists, then update only if the output
@@ -381,7 +399,7 @@ void SessionController::setupActions()
 
     // Open Browser
     action = collection->addAction("open-browser");
-    action->setText( i18n("Open Browser Here") );
+    action->setText( i18n("Open File Manager") );
     action->setIcon( KIcon("system-file-manager") );
     connect( action, SIGNAL(triggered()), this, SLOT(openBrowser()) );
 
@@ -415,22 +433,40 @@ void SessionController::setupActions()
     action->setShortcut( QKeySequence(Qt::CTRL+Qt::ALT+Qt::Key_S) );
     connect( action , SIGNAL(triggered()) , this , SLOT(renameSession()) );
 
-    // Copy Input To
-    action = collection->addAction("copy-input-to");
-    action->setText(i18n("Copy Input To..."));
-    connect( action , SIGNAL(triggered()) , this , SLOT(copyInputTo()) );
+    // Copy Input To -> All Tabs in Current Window
+    _copyToAllTabsAction = collection->addAction("copy-input-to-all-tabs");
+    _copyToAllTabsAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_Comma) );
+    _copyToAllTabsAction->setText(i18n("&All Tabs in Current Window") );
+    _copyToAllTabsAction->setCheckable(true);
+    connect( _copyToAllTabsAction , SIGNAL(triggered()) , this , SLOT(copyInputToAllTabs()) );
 
-    // Clear and Clear+Reset
-    action = collection->addAction("clear");
-    action->setText( i18n("C&lear Display") );
-    action->setIcon( KIcon("edit-clear") );
-    connect( action , SIGNAL(triggered()) , this , SLOT(clear()) );
+    // Copy Input To -> Select Tabs
+    _copyToSelectedAction = collection->addAction("copy-input-to-selected-tabs");
+    _copyToSelectedAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_Period) );
+    _copyToSelectedAction->setText( i18n("&Select Tabs...") );
+    _copyToSelectedAction->setCheckable(true);
+    connect( _copyToSelectedAction , SIGNAL(triggered()) , this , SLOT(copyInputToSelectedTabs()) );
 
+    // Copy Input To -> None
+    _copyToNoneAction = collection->addAction("copy-input-to-none");
+    _copyToNoneAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_Slash) );
+    _copyToNoneAction->setText( i18n("&None") );
+    _copyToNoneAction->setCheckable(true);
+    _copyToNoneAction->setChecked(true);
+    connect( _copyToNoneAction , SIGNAL(triggered()) , this , SLOT(copyInputToNone()) );
+
+    // Clear+Reset
     action = collection->addAction("clear-and-reset");
     action->setText( i18n("Clear && Reset") );
     action->setIcon( KIcon("edit-clear-history") );
     connect( action , SIGNAL(triggered()) , this , SLOT(clearAndReset()) );
 
+    action = collection->addAction("zmodem-upload");
+    action->setText( i18n( "&ZModem Upload..." ) );
+    action->setIcon( KIcon("document-open") );
+    action->setShortcut( QKeySequence(Qt::CTRL+Qt::ALT+Qt::Key_U) );
+    connect( action , SIGNAL(triggered()) , this , SLOT(zmodemUpload()) );
+    
     // Monitor
     toggleAction = new KToggleAction(i18n("Monitor for &Activity"),this);
     toggleAction->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_A) );
@@ -493,10 +529,6 @@ void SessionController::setupActions()
     action->setIcon( KIcon("configure") );
     connect( action , SIGNAL(triggered()) , this , SLOT(showHistoryOptions()) );
 
-    action = collection->addAction("clear-history");
-    action->setText( i18n("Clear Scrollback") );
-    connect( action , SIGNAL(triggered()) , this , SLOT(clearHistory()) );
-
     action = collection->addAction("clear-history-and-reset");
     action->setText( i18n("Clear Scrollback && Reset") );
     action->setShortcut( QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_X) );
@@ -579,9 +611,9 @@ void SessionController::saveSession()
 }
 bool SessionController::confirmClose() const
 {
-    if (_session->isChildActive())
+    if (_session->isForegroundProcessActive())
     {
-        QString title = _session->childName();
+        QString title = _session->foregroundProcessName();
       
         // hard coded for now.  In future make it possible for the user to specify which programs
         // are ignored when considering whether to display a confirmation
@@ -630,7 +662,68 @@ void SessionController::pasteSelection()
 {
     _view->pasteSelection();
 }
-void SessionController::copyInputTo()
+static const KXmlGuiWindow* findWindow(const QObject* object)
+{
+    // Walk up the QObject hierarchy to find a KXmlGuiWindow.
+    while(object != NULL) {
+        const KXmlGuiWindow* window = dynamic_cast<const KXmlGuiWindow*>(object);
+        if(window != NULL) {
+            return(window);
+        }
+        object = object->parent();
+    }
+    return(NULL);
+}
+
+static bool hasTerminalDisplayInSameWindow(const Session* session, const KXmlGuiWindow* window)
+{
+    // Iterate all TerminalDisplays of this Session ...
+    QListIterator<TerminalDisplay*> terminalDisplayIterator(session->views());
+    while(terminalDisplayIterator.hasNext()) {
+        const TerminalDisplay* terminalDisplay = terminalDisplayIterator.next();
+        // ... and check whether a TerminalDisplay has the same
+        // window as given in the parameter
+        if(window == findWindow(terminalDisplay)) {
+            return(true);    
+        }
+    }
+    return(false);
+}
+
+void SessionController::copyInputToAllTabs()
+{
+    if(!_copyToGroup) {
+        _copyToGroup = new SessionGroup(this);
+    }
+
+    // Find our window ...
+    const KXmlGuiWindow* myWindow = findWindow(_view);
+
+    QSet<Session*> group =
+       QSet<Session*>::fromList(SessionManager::instance()->sessions());
+    for(QSet<Session*>::iterator iterator = group.begin();
+        iterator != group.end(); ++iterator) {
+        Session* session = *iterator;
+ 
+        // First, ensure that the session is removed
+        // (necessary to avoid duplicates on addSession()!)
+        _copyToGroup->removeSession(session);
+
+        // Add current session if it is displayed our window
+        if(hasTerminalDisplayInSameWindow(session, myWindow)) {
+            _copyToGroup->addSession(session);
+        }
+    }
+    _copyToGroup->setMasterStatus(_session, true);
+    _copyToGroup->setMasterMode(SessionGroup::CopyInputToAll);
+    
+    snapshot();
+    _copyToAllTabsAction->setChecked(true);
+    _copyToSelectedAction->setChecked(false);
+    _copyToNoneAction->setChecked(false);
+}
+
+void SessionController::copyInputToSelectedTabs()
 {
     if (!_copyToGroup)
     {
@@ -667,11 +760,41 @@ void SessionController::copyInputTo()
                 _copyToGroup->removeSession(session);
         }
 
-        snapshot();
+        _copyToGroup->setMasterStatus(_session, true);
+        _copyToGroup->setMasterMode(SessionGroup::CopyInputToAll);
+        snapshot();        
     }
 
     delete dialog;
+    _copyToAllTabsAction->setChecked(false);
+    _copyToSelectedAction->setChecked(true);
+    _copyToNoneAction->setChecked(false);
 }
+
+void SessionController::copyInputToNone()
+{
+    if (!_copyToGroup)      // No 'Copy To' is active
+        return;
+
+    QSet<Session*> group =
+       QSet<Session*>::fromList(SessionManager::instance()->sessions());
+    for(QSet<Session*>::iterator iterator = group.begin();
+        iterator != group.end(); ++iterator) {
+        Session* session = *iterator;
+ 
+        if(session != _session) {
+            _copyToGroup->removeSession(*iterator);
+        }
+    }
+    delete _copyToGroup;
+    _copyToGroup = NULL;
+    snapshot();
+    
+    _copyToAllTabsAction->setChecked(false);
+    _copyToSelectedAction->setChecked(false);
+    _copyToNoneAction->setChecked(true);
+}
+
 void SessionController::clear()
 {
     Emulation* emulation = _session->emulation();
@@ -845,8 +968,8 @@ void SessionController::showHistoryOptions()
     else
         dialog->setMode( HistorySizeDialog::NoHistory );
 
-    connect( dialog , SIGNAL(optionsChanged(int,int)) ,
-             this , SLOT(scrollBackOptionsChanged(int,int)) );
+    connect( dialog , SIGNAL(optionsChanged(int,int,bool)) ,
+             this , SLOT(scrollBackOptionsChanged(int,int,bool)) );
 
     dialog->show();
 }
@@ -855,7 +978,7 @@ void SessionController::sessionResizeRequest(const QSize& size)
     kDebug(1211) << "View resize requested to " << size;
     _view->setSize(size.width(),size.height());
 }
-void SessionController::scrollBackOptionsChanged( int mode , int lines )
+void SessionController::scrollBackOptionsChanged(int mode, int lines, bool saveToCurrentProfile )
 {
     switch (mode)
     {
@@ -863,12 +986,31 @@ void SessionController::scrollBackOptionsChanged( int mode , int lines )
             _session->setHistoryType( HistoryTypeNone() );
             break;
          case HistorySizeDialog::FixedSizeHistory:
-            _session->setHistoryType( HistoryTypeBuffer(lines) );
+            _session->setHistoryType( CompactHistoryType(lines) );
             break;
          case HistorySizeDialog::UnlimitedHistory:
              _session->setHistoryType( HistoryTypeFile() );
             break;
     }
+    if (saveToCurrentProfile)
+    {
+        Profile::Ptr profile = SessionManager::instance()->sessionProfile(_session);
+
+        switch (mode)
+        {
+            case HistorySizeDialog::NoHistory:
+                profile->setProperty(Profile::HistoryMode , Profile::DisableHistory);
+                break;
+            case HistorySizeDialog::FixedSizeHistory:
+                profile->setProperty(Profile::HistoryMode , Profile::FixedSizeHistory);
+                profile->setProperty(Profile::HistorySize , lines);
+                break;
+            case HistorySizeDialog::UnlimitedHistory:
+                profile->setProperty(Profile::HistoryMode , Profile::UnlimitedHistory);
+                break;
+        }
+        SessionManager::instance()->changeProfile(profile, profile->setProperties());
+    }
 }
 
 void SessionController::saveHistory()
@@ -913,13 +1055,25 @@ void SessionController::monitorSilence(bool monitor)
 {
     _session->setMonitorSilence(monitor);
 }
+void SessionController::updateSessionIcon()
+{
+    // Visualize that the session is broadcasting to others
+    if (_copyToGroup && _copyToGroup->sessions().count() > 1) {
+        // Master Mode: set different icon, to warn the user to be careful
+        setIcon(KIcon("emblem-important"));
+    }
+    else {
+        // Not in Master Mode: use normal icon
+        setIcon( _sessionIcon );
+    }
+}
 void SessionController::sessionTitleChanged()
 {
         if ( _sessionIconName != _session->iconName() )
         {
             _sessionIconName = _session->iconName();
             _sessionIcon = KIcon( _sessionIconName );
-            setIcon( _sessionIcon );
+            updateSessionIcon();
         }
 
         QString title = _session->title(Session::DisplayedTitleRole);
@@ -1018,10 +1172,70 @@ void SessionController::sessionStateChanged(int state)
             _sessionIcon = KIcon( _sessionIconName );
         }
 
-        setIcon( _sessionIcon );
+        updateSessionIcon();
     }
 }
 
+void SessionController::zmodemDownload()
+{
+    QString zmodem = KGlobal::dirs()->findExe("rz");
+    if(zmodem.isEmpty()) {
+       zmodem = KGlobal::dirs()->findExe("lrz");
+    }
+    if(!zmodem.isEmpty()) {
+        const QString path = KFileDialog::getExistingDirectory(
+                                QString(), _view,
+                                i18n("Save ZModem Download to..."));
+
+        if(!path.isEmpty()) {
+            _session->startZModem(zmodem, path, QStringList());
+            return;
+        }
+    }
+    else {
+        KMessageBox::error(_view,
+          i18n("<p>A ZModem file transfer attempt has been detected, "
+               "but no suitable ZModem software was found on this system.\n"
+               "<p>You may wish to install the 'rzsz' or 'lrzsz' package.\n"));
+    }
+    _session->cancelZModem();
+    return;
+}
+
+void SessionController::zmodemUpload()
+{
+    if(_session->isZModemBusy()) {
+      KMessageBox::sorry(_view,
+         i18n("<p>The current session already has a ZModem file transfer in progress."));
+      return;
+    }
+    QString zmodem = KGlobal::dirs()->findExe("sz");
+    if(zmodem.isEmpty()) {
+       zmodem = KGlobal::dirs()->findExe("lsz");
+    }
+    if(zmodem.isEmpty()) {
+        KMessageBox::sorry(_view,
+           i18n("<p>No suitable ZModem software was found on this system.\n"
+                "<p>You may wish to install the 'rzsz' or 'lrzsz' package.\n"));
+        return;
+    }
+
+    QStringList files = KFileDialog::getOpenFileNames(KUrl(), QString(), _view,
+                           i18n("Select Files for ZModem Upload"));
+    if(!files.isEmpty()) {
+        _session->startZModem(zmodem, QString(), files);
+    }
+}
+
+bool SessionController::isKonsolePart() const
+{
+    // Check to see if we are being called from Konsole or a KPart
+    if (QString(qApp->metaObject()->className()) == "Konsole::Application")
+        return false;
+    else
+        return true;
+}
+
 SessionTask::SessionTask(QObject* parent)
     :  QObject(parent)
     ,  _autoDelete(false)
@@ -1182,11 +1396,11 @@ void SaveHistoryTask::jobResult(KJob* job)
         KMessageBox::sorry( 0 , i18n("A problem occurred when saving the output.\n%1",job->errorString()) );
     }
 
-    SaveJob& info = _jobSession[job];
+    TerminalCharacterDecoder * decoder = _jobSession[job].decoder;
 
     _jobSession.remove(job);
 
-    delete info.decoder;
+    delete decoder;
 
     // notify the world that the task is done
     emit completed(true);
@@ -1225,7 +1439,9 @@ void SearchHistoryTask::executeOnScreenWindow( SessionPtr session , ScreenWindow
     {
         int pos = -1;
         const bool forwards = ( _direction == ForwardsSearch );
-        const int startLine = selectionLine + window->currentLine() + ( forwards ? 1 : -1 );
+        int startLine = selectionLine + window->currentLine() + ( forwards ? 1 : -1 );
+        // Temporary fix for #205495
+        if (startLine < 0) startLine = 0;
         const int lastLine = window->lineCount() - 1;
         QString string;
 
@@ -1373,11 +1589,3 @@ QRegExp SearchHistoryTask::regExp() const
 
 #include "SessionController.moc"
 
-/*
-  Local Variables:
-  mode: c++
-  c-file-style: "stroustrup"
-  indent-tabs-mode: nil
-  tab-width: 4
-  End:
-*/
