@@ -428,20 +428,19 @@ TabbedViewContainer::TabbedViewContainer(NavigationPosition position , QObject*
     _stackWidget = new QStackedWidget();
     _tabBar = new ViewContainerTabBar(_containerWidget,this);
     _tabBar->setDrawBase(true);
+    _tabBar->setDocumentMode(true);
+    _tabBar->setFocusPolicy(Qt::NoFocus);
 
-    const int cornerButtonWidth = 50;
-    _newTabButton = new KPushButton(KIcon("tab-new"),QString(),_containerWidget);
-    // The button width here is hard coded, it would be better to use the value from
-    // the current style (see QTabWidget::setUpLayout())
-    _newTabButton->setFixedWidth(cornerButtonWidth);
-    _newTabButton->setFlat(true);
+    _newTabButton = new QToolButton(_containerWidget);
+    _newTabButton->setIcon(KIcon("tab-new"));
+    _newTabButton->adjustSize();
     // new tab button is initially hidden, it will be shown when setFeatures() is called
     // with the QuickNewView flag enabled
     _newTabButton->setHidden(true);
-   
-    _closeTabButton = new KPushButton(KIcon("tab-close"),QString(),_containerWidget);
-    _closeTabButton->setFixedWidth(cornerButtonWidth);
-    _closeTabButton->setFlat(true);
+
+    _closeTabButton = new QToolButton(_containerWidget);
+    _closeTabButton->setIcon(KIcon("tab-close"));
+    _closeTabButton->adjustSize();
     _closeTabButton->setHidden(true);
 
     connect( _tabBar , SIGNAL(currentChanged(int)) , this , SLOT(currentTabChanged(int)) );
@@ -488,7 +487,7 @@ TabbedViewContainer::TabbedViewContainer(NavigationPosition position , QObject*
 }
 void TabbedViewContainer::setNewViewMenu(QMenu* menu)
 {
-  _newTabButton->setDelayedMenu(menu); 
+  _newTabButton->setMenu(menu);
 }
 ViewContainer::Features TabbedViewContainer::supportedFeatures() const
 { 
@@ -814,6 +813,7 @@ void StackedViewContainer::removeViewWidget( QWidget* view )
     const int index = _stackWidget->indexOf(view);
 
     Q_ASSERT( index != -1);
+    Q_UNUSED(index);
 
     _stackWidget->removeWidget(view);
 }
