@@ -35,6 +35,7 @@
 #include <KLocale>
 #include <KToggleAction>
 #include <KXMLGUIFactory>
+#include <QStringList>
 
 // Konsole
 #include "ColorScheme.h"
@@ -45,6 +46,8 @@
 #include "SessionManager.h"
 #include "ViewContainer.h"
 #include "ViewSplitter.h"
+#include "konsoleadaptor.h"
+#include "Profile.h"
 
 using namespace Konsole;
 
@@ -85,6 +88,11 @@ ViewManager::ViewManager(QObject* parent , KActionCollection* collection)
             SLOT(profileChanged(Profile::Ptr)) );
     connect( SessionManager::instance() , SIGNAL(sessionUpdated(Session*)) , this,
             SLOT(updateViewsForSession(Session*)) );
+
+    //prepare DBus communication
+    new KonsoleAdaptor(this);
+    QDBusConnection::sessionBus().registerObject(QLatin1String("/Konsole"), this);
+
 }
 
 ViewManager::~ViewManager()
@@ -225,14 +233,14 @@ void ViewManager::setupActions()
     // keyboard shortcut only actions
     KShortcut nextViewShortcut = nextViewAction->shortcut();
     nextViewShortcut.setPrimary( QKeySequence(Qt::SHIFT+Qt::Key_Right) );
-    nextViewShortcut.setAlternate( QKeySequence(Qt::CTRL+Qt::Key_PageUp) );
+    nextViewShortcut.setAlternate( QKeySequence(Qt::CTRL+Qt::Key_PageDown) );
     nextViewAction->setShortcut(nextViewShortcut); 
     connect( nextViewAction, SIGNAL(triggered()) , this , SLOT(nextView()) );
     _viewSplitter->addAction(nextViewAction);
 
     KShortcut previousViewShortcut = previousViewAction->shortcut();
     previousViewShortcut.setPrimary( QKeySequence(Qt::SHIFT+Qt::Key_Left) );
-    previousViewShortcut.setAlternate( QKeySequence(Qt::CTRL+Qt::Key_PageDown) );
+    previousViewShortcut.setAlternate( QKeySequence(Qt::CTRL+Qt::Key_PageUp) );
     previousViewAction->setShortcut(previousViewShortcut);
     connect( previousViewAction, SIGNAL(triggered()) , this , SLOT(previousView()) );
     _viewSplitter->addAction(previousViewAction);
@@ -345,10 +353,10 @@ void ViewManager::sessionFinished()
 
     if ( _sessionMap[qobject_cast<TerminalDisplay*>(activeView())] == session )
     {
-        // switch to the previous view before deleting the session views to prevent flicker 
+        // switch to the next view before deleting the session views to prevent flicker 
         // occurring as a result of an interval between removing the active view and switching
-        // to the previous view
-        previousView();
+        // to the next view
+        nextView();
     }
 
     Q_ASSERT(session);
@@ -648,6 +656,15 @@ void ViewManager::setNavigationMethod(NavigationMethod method)
 
     if ( collection )
     {
+        // FIXME: The following disables certain actions for the KPart that it
+        // doesn't actually have a use for, to avoid polluting the action/shortcut
+        // namespace of an application using the KPart (otherwise, a shortcut may
+        // be in use twice, and the user gets to see an "ambiguous shortcut over-
+        // load" error dialog). However, this approach sucks - it's the inverse of
+        // what it should be. Rather than disabling actions not used by the KPart,
+        // a method should be devised to only enable those that are used, perhaps
+        // by using a separate action collection.
+
         QAction* action;
 
         action = collection->action( "next-view" );
@@ -664,6 +681,12 @@ void ViewManager::setNavigationMethod(NavigationMethod method)
 
         action = collection->action( "rename-session" );
         if ( action ) action->setEnabled( _navigationMethod != NoNavigation );
+
+        action = collection->action( "move-view-left" );
+        if ( action ) action->setEnabled( _navigationMethod != NoNavigation );
+
+        action = collection->action( "move-view-right" );
+        if ( action ) action->setEnabled( _navigationMethod != NoNavigation );
     }
 }
 
@@ -802,6 +825,9 @@ void ViewManager::applyProfile(TerminalDisplay* view , const Profile::Ptr info,
     bool blinkingText = info->property<bool>(Profile::BlinkingTextEnabled);
     view->setBlinkingTextEnabled(blinkingText);
 
+    bool tripleClickMode = info->property<bool>(Profile::TripleClickMode);
+    view->setTripleClickMode(tripleClickMode ? TerminalDisplay::SelectForwardsFromCursor : TerminalDisplay::SelectWholeLine);
+
     bool bidiEnabled = info->property<bool>(Profile::BidiRenderingEnabled);
     view->setBidiEnabled(bidiEnabled);
 
@@ -929,6 +955,15 @@ void ViewManager::restoreSessions(const KConfigGroup& group)
         _viewSplitter->activeContainer()->setActiveView(display);
         display->setFocus(Qt::OtherFocusReason);
     }
+
+    if (ids.isEmpty()) // Session file is unusable, start default Profile
+    {
+        Profile::Ptr profile = SessionManager::instance()->defaultProfile();
+        Session* session = SessionManager::instance()->createSession(profile);
+        createView(session);
+        if (!session->isRunning())
+            session->run();
+    }
 }
 
 uint qHash(QPointer<TerminalDisplay> display)
@@ -936,13 +971,89 @@ uint qHash(QPointer<TerminalDisplay> display)
     return qHash((TerminalDisplay*)display);
 }
 
+int ViewManager::sessionCount()
+{
+    return this->_sessionMap.size();
+}
+
+int ViewManager::currentSession()
+{
+    QHash<TerminalDisplay*,Session*>::iterator i;
+    for (i = this->_sessionMap.begin(); i != this->_sessionMap.end(); ++i)
+        if (i.key()->isVisible())
+            return i.value()->sessionId();
+    return -1;
+}
+
+int ViewManager::newSession()
+{
+    Profile::Ptr profile = SessionManager::instance()->defaultProfile();
+    Session* session = SessionManager::instance()->createSession(profile);
+
+    this->createView(session);
+    session->run();
+
+    return session->sessionId();
+}
+
+int ViewManager::newSession(QString profile, QString directory)
+{
+    QList<Profile::Ptr> profilelist = SessionManager::instance()->loadedProfiles();
+    QList<Profile::Ptr>::iterator i = profilelist.begin();
+
+    Profile::Ptr profileptr = SessionManager::instance()->defaultProfile();
+
+    while (i != profilelist.end() )
+    {
+        Profile::Ptr ptr = *i;
+        if ( ptr->name().compare(profile) == 0)
+            profileptr = ptr;
+        i++;
+    }
+
+    Session* session = SessionManager::instance()->createSession(profileptr);
+    session->setInitialWorkingDirectory(directory);
+
+    this->createView(session);
+    session->run();
+
+    return session->sessionId();
+}
+
+QStringList ViewManager::profileList()
+{
+    QList<Profile::Ptr> profilelist = SessionManager::instance()->loadedProfiles();
+    QList<Profile::Ptr>::iterator i = profilelist.begin();
+    QStringList list;
+    while (i != profilelist.end() )
+    {
+        Profile::Ptr ptr = *i;
+        list.push_back(ptr->name());
+        i++;
+    }
+
+    return list;
+}
+
+void ViewManager::nextSession()
+{
+    this->nextView();
+}
+
+void ViewManager::prevSession()
+{
+    this->previousView();
+}
+
+void ViewManager::moveSessionLeft()
+{
+    this->moveActiveViewLeft();
+}
+
+void ViewManager::moveSessionRight()
+{
+    this->moveActiveViewRight();
+}
+
 #include "ViewManager.moc"
 
-/*
-  Local Variables:
-  mode: c++
-  c-file-style: "stroustrup"
-  indent-tabs-mode: nil
-  tab-width: 4
-  End:
-*/
